@@ -1,22 +1,14 @@
 from abc import abstractmethod
+from typing import Iterable
 
-import torch
+import numpy as np
+from torch import nn
 
 
-class _DCCA_base(torch.nn.Module):
+class _DCCA_base(nn.Module):
     def __init__(self, latent_dims: int):
-        super(_DCCA_base, self).__init__()
+        super().__init__()
         self.latent_dims = latent_dims
-        self.schedulers = [None]
-
-    @abstractmethod
-    def update_weights(self, *args):
-        """
-        A complete update of the weights used every batch
-        :param args: batches for each view separated by commas
-        :return:
-        """
-        pass
 
     @abstractmethod
     def forward(self, *args):
@@ -26,5 +18,5 @@ class _DCCA_base(torch.nn.Module):
         """
         pass
 
-    def post_transform(self, *z_list, train=False):
+    def post_transform(self, *z_list, train=False) -> Iterable[np.ndarray]:
         return z_list
